/*       -*- C++ -*- */
/** \file
********************************************************************
* $URL: svn+ssh://nalw02/home/alg/schlemme/svn/cextools/trunk/include/cexfile/concept_base.h $
* $Id: concept_base.h 15 2010-06-25 19:53:27Z schlemme $
* $Date: 2010-06-25 21:53:27 +0200 (Fr, 25. Jun 2010) $
* $Revision: 15 $
* \author Tobias Schlemmer <keinstein@users.berlios.de>
* $Author: schlemme $ 
* \license LGPL
* 
* Concept base class.
********************************************************************
*/	  
#ifndef CEXFILE_CONCEPT_BASE_H
#define CEXFILE_CONCEPT_BASE_H

#include <vector>
#include "cexfile/point.h"
#include "cexfile/object_base.h"


namespace cexfile {
	/// a generalized concept
	class concept_base:public object_base {
	protected:
		/// position of the %object
		point position;
	public:
		/// default constructor
		concept_base():position(),object_base() {}
    
		/// standard constructor
		/** \param Node XML node to be parsed
		 */
		concept_base(const xmlpp::Node * Node);
    
		/// return the position
		/** \return point describing the position
		 */
		const point& get_position() const {return position;}

		/// linear ordering concept_base %objects
		/** \param c other %object to be compared
		    \retval true if we are lower than \p c
		*/
		bool operator < (const concept_base & c) {
			return (object_base) *this < (object_base) c;
		}
	};
  
	/// a set of concept_base
	typedef std::vector<concept_base> concept_base_s;

	/// add a concept_base %object to the set
	/** \relates concept_base_s
	    \relates concept_base
	    \param s set to be appended to
	    \param c concept_base %object to be added
	    \return reference to \p s after appending 
	*/
	inline concept_base_s& operator += (concept_base_s& s, const concept_base c) {
		s.push_back(c);
		return s;
	}

	inline concept_base::concept_base(const xmlpp::Node * Node):object_base(Node) {
		if (!Node) throw xmlpp::validity_error("Empty parent Node for concept base.");

#ifdef DEBUG
		std::cerr << "Concept base: " << Node -> get_path() << std::endl;
#endif

		position = point(Node);
	}

}


#endif /* CEXFILE_CONCEPT_BASE_H */
