/*       -*- C++ -*- */
/** \file
********************************************************************
* $URL: svn+ssh://nalw02/home/alg/schlemme/svn/cextools/trunk/include/cexfile/edge.h $
* $Id: edge.h 15 2010-06-25 19:53:27Z schlemme $
* $Date: 2010-06-25 21:53:27 +0200 (Fr, 25. Jun 2010) $
* $Revision: 15 $
* \author Tobias Schlemmer <keinstein@users.berlios.de>
* $Author: schlemme $ 
* \license LGPL
* 
* Edge class.
********************************************************************
*/	  
#ifndef CEXFILE_EDGE_H
#define CEXFILE_EDGE_H

#include "cexfile/concept.h"
#include "cexfile/rect.h"


namespace cexfile {
	/// edge class.
	class edge:public rect {
	private:
		/// concepts (or similar objects) which denote the end points
		const concept_base *from, *to;

		/// sets the concepts of the end points
		/** This function initializes the coordinates from the concepts.
		    \param f start concept
		    \param t end concept
		*/
		void set_concepts(const concept_base * f, const concept_base * t)
			{
				assert(f);
				assert(t);
				from = f;
				to = t;
				start = f->get_position(); 
				end = t->get_position();
#ifdef DEBUG
				std::cerr << "Edge: " << start << "-" << end << std::endl;
#endif
			}
	public:

		/// default constructor
		edge():from(NULL),to(NULL) {}

		/// create edge between two concepts
		/** \param f reference to starting concept
		    \param t reference to end concept
		*/
		edge(const concept_s::reference f, const concept_s::reference t) {
			set_concepts(&f,&t);
		}

		/// create an edge between two generalized concepts
		/** \param f pointer to the start of the edge
		    \param t pointer to the end of the edge
		*/
		edge(const concept_base * f, const concept_base * t) {
			set_concepts(f,t);
		}
    
		/// create an edge between two generalized concepts
		/** \param f pointer to the start of the edge
		    \param t pointer to the end of the edge
		*/
		edge(const concept_base & f, const concept_base & t) {
			set_concepts(&f,&t);
		}

		/// create an edge between two nodes in the concept lattice order
		/** \param f reference to the start of the edge
		    \param t reference to the end of the edge
		*/
		edge(const concept_sibling& f, const concept_sibling& t){
			set_concepts(f.get_refp(), t.get_refp());
		}

		/// get the start of the edge
		/** \return pointer to the start concept */
		const concept_base * getfrom() const {return from;}

		/// get the end of the edge
		/** \return pointer to the end concept */
		const concept_base * getto() const {return to;}
	};
  
	/// a set of edges
	typedef std::list<edge> edge_s;

	/// append an edge to the set of edges
	/** \relates edge_s
	    \relates edge
	    \param s set of edges
	    \param e edge to be appended
	    \return reference to \p s
	*/
	inline edge_s& operator += (edge_s& s, const edge e) {
		s.push_back(e);
		return s;
	}

}


#endif /* CEXFILE_EDGE_H */
