/*       -*- C++ -*- */
/** \file
 ********************************************************************
 * $URL: svn+ssh://nalw02/home/alg/schlemme/svn/cextools/trunk/include/cexfile/label.h $
 * $Id: label.h 12 2008-02-26 15:45:35Z schlemme $
 * $Date: 2008-02-26 16:45:35 +0100 (Di, 26 Feb 2008) $
 * $Revision: 12 $
 * \author Tobias Schlemmer <keinstein@users.berlios.de>
 * $Author: schlemme $ 
 * \license LGPL
 * 
 * Label class.
 ********************************************************************
 */	  
#ifndef CEXFILE_LABEL_H
#define CEXFILE_LABEL_H

#include "cexfile/concept_base.h"
#include "cexfile/concept.h"
#include "cexfile/edge.h"
#include "cexfile/attribute.h"

namespace cexfile {
  /// label size calculation function
  /** \deprecated there exist more flexible mechanisms now.
      Some drawing libraries don't support multiline strings anyway.
  */
  class label_sizer {
    virtual rect get_size(Glib::ustring s) { return rect(); }
  };

  /// a label to be drawn near a concept
  class label:public concept_base {
    /// The background rectangle
    rect area;

    /// edge linking the concept node and the text rectangle
    edge link;

    /// concept the label belongs to
    const concept * concept_ref;

    /// text rows of the label
    std::vector<Glib::ustring> names;
  public:

    /// currently unused: user data
    void * userdata;

    /// default constructor
    label():concept_base(), concept_ref(NULL) {}

    /// standard constructor
    /** \param Node root node of the label 
     */
    label(const xmlpp::Node * Node):concept_base(Node), concept_ref(NULL) {}

    /// associate the label with a concept
    /** \param c concept the label belongs to 
     */
    void update_concept(const concept & c) {
      concept_ref = &c;
      assert(attributes == c.get_intent());
      link=edge(this,concept_ref);
    }

    /// use the object names of our concept as label text
    void set_object_names() {
      const object_rs & objects = *(concept_ref -> get_objects());
      names.clear();
      if (!objects.size()) {
	std::cerr << "Warning: got empty concept label" << std::endl;
	return; 
      }
      names.reserve(objects.size());
      for (object_rs::iterator i = objects.begin(); 
	   i != objects.end(); i++) {
	names.push_back((Glib::ustring)*(*i));
      }
    }

    /// use the attribute names of our concept as label text
    /** \note The attributes are stored inside a concetp 
	as indices to the attribute set only. So we need the 
	complete set of attibutes seperately.
	\param attribs Set of the attributes
     */
    void set_attribute_names(const attribute_s & attribs) {
      attribute_s &attributes = const_cast<attribute_s &> (attribs);
      const intent & attr = concept_ref -> get_attributes();
      names.clear(); 
      if (!attr.size()) {
	std::cerr << "Warning: got empty attribute label" << std::endl;
	return; 
      }
      names.reserve(attr.size());
      for (intent::iterator i = attr.begin();
	   i != attr.end(); i++) {
	names.push_back( (Glib::ustring) attributes[*i]);
      }
    }
    
    /// set the label text as a single string
    /** \param s string to be set
     */
    void set_string(Glib::ustring s) { 
      names.clear();
      names.push_back(s); 
    }

    /// returns the label strings
    /** This function returns the label strings.
	\return reference to a vector of strings to be printed at the label.
    */
    const std::vector<Glib::ustring> & get_strings () const { return names; }

    /// returns the edge linking the label to the concept node
    /** \return reference to the linking edge 
     */
    const edge & get_link() const { return link; }
  };

  /// a set of labels
  typedef std::vector<label> label_s;

  /// adds a label to a set of labels
  /** \relates label_s
      \relates label
      \param s set of labels, to which we add the label
      \param c label to be added
      \return reference to the label set
  */
  inline label_s& operator += (label_s& s, const label c) {
    s.push_back(c);
    return s;
  }


}

#endif /* CEXFILE_LABEL_H */
