/*       -*- C++ -*- */
/** \file
********************************************************************
* $URL: svn+ssh://nalw02/home/alg/schlemme/svn/cextools/trunk/include/cexfile/lattice.h $
* $Id: lattice.h 15 2010-06-25 19:53:27Z schlemme $
* $Date: 2010-06-25 21:53:27 +0200 (Fr, 25. Jun 2010) $
* $Revision: 15 $
* \author Tobias Schlemmer <keinstein@users.berlios.de>
* $Author: schlemme $ 
* \license LGPL
* 
* Lattice class.
********************************************************************
*/	  
#ifndef CEXFILE_LATTICE_H
#define CEXFILE_LATTICE_H

#include <iostream>
#include <vector>
#include "cexfile/diagram.h"

namespace cexfile {
	/// concept lattice class
	class lattice {
		/// XML root node of the lattice
		const xmlpp::Node * node;
		/// context of the lattice
		context_s contexts;
		/// diagrams built according to the lattice
		diagram_s diagrams;


	public:
		/// default constructor
		lattice():node(NULL) {}
		/// standard constructor
		/** \param Node root XML node of the lattice
		    \param c set of contexts
		*/
		lattice(const xmlpp::Node * Node, context_s & c);

		/// return the diagrams
		/** \return reference to the diagram set of the latttice 
		 */
		const diagram_s & get_diagrams() const { return diagrams; }

		/// output some statistics
		/** \param o output stream
		    \return reference to the output stream
		*/ 
		std::ostream & statistics (std::ostream & o) const;
	};
  
	/// output operator
	/** this operator calls lattice::statistics() to output
	    some statistics to the stream
	    \relates lattice
	    \param o output stream
	    \param l lattice whose statistics shall be written
	    \return reference to the output stream
	*/
	inline std::ostream & operator << (std::ostream & o, const lattice & l) {
		return l.statistics (o);
	}

	/// a set of lattices
	typedef std::vector<lattice> lattice_s;

	/// add a lattice to the set of lattices
	/** \relates lattice
	    \param s set of lattices to be expanded
	    \param c lattice to be added
	    \return reference to the set \p s
	*/
	inline lattice_s& operator+=(lattice_s& s, lattice c) {
		s.push_back(c);
		return s;
	}

	inline lattice::lattice(const xmlpp::Node * Node, 
				context_s & c):node(Node),
					       contexts(c)
	{
		if (!Node) throw xmlpp::validity_error("Empty parent Node for lattices.");

#ifdef DEBUG
		std::cerr << "Lattice: " << Node -> get_path() << std::endl;
#endif
    
		xmlpp::NodeSet s = Node -> find ("LineDiagram");
		for (xmlpp::NodeSet::iterator i=s.begin(); i!=s.end(); i++) {
			diagrams += diagram(*i,c);
		}

    
	}

	inline std::ostream & lattice::statistics (std::ostream & o ) const {
		o << diagrams.size() << " diagrams:" << std::endl;
		int j = 1;
		for (diagram_s::const_iterator i=diagrams.begin(); i!= diagrams.end();i++) {
			o << "Diagram " << j++ << std::endl;
			o << *i ;
		}
		o << " diagrams end" << std::endl;
		return o;
	}
  
}

#endif /* CEXFILE_LATTICE_H */
