/*       -*- C++ -*- */
/** \file
 ********************************************************************
 * $URL: svn+ssh://nalw02/home/alg/schlemme/svn/cextools/trunk/include/cexfile/object_label.h $
 * $Id: object_label.h 12 2008-02-26 15:45:35Z schlemme $
 * $Date: 2008-02-26 16:45:35 +0100 (Di, 26 Feb 2008) $
 * $Revision: 12 $
 * \author Tobias Schlemmer <keinstein@users.berlios.de>
 * $Author: schlemme $ 
 * \license LGPL
 * 
 * Object label class.
 ********************************************************************
 */	  
#ifndef CEXFILE_OBJECT_LABEL_H
#define CEXFILE_OBJECT_LABEL_H

#include "cexfile/label.h"
#include "cexfile/object.h"

namespace cexfile {
  /// a single object label
  class object_label:protected label {
    /// identifier of the corresponding object
    object::idtype id;
  public:
    /// default constructor
    object_label():label(),id(object::noId) {}

    /// standard constructor
    /** \param Node XML root node of the label
     */
    object_label(const xmlpp::Node * Node);

    /// get the object id of the corresponding object
    /** \return object id of corresponding object 
     */
    object::idtype get_id() const {return id;}
  };
  
  /// set mapping the identifiers of objects to the object labels
  typedef std::map<object::idtype,object_label> object_label_s;

  /// add an object label to the set 
  /** \relates object_label_s
      \relates object_label
      \param s set of object labels
      \param c object label to be added
      \return reference to the set after adding the label
  */
  inline object_label_s& operator += (object_label_s& s, 
					 const object_label c) {
    s[c.get_id()]=c;
    return s;
  }

  inline object_label::object_label (const xmlpp::Node * Node)
    :label()
  {
    if (!Node) throw xmlpp::validity_error("Empty object label node.");

#ifdef DEBUG
    std::cerr << "Object Label: " << Node -> get_path() << std::endl;
#endif

    position = point(Node);
    id = atoi(parse_single_node_attribute(Node,OBJECT_REFERENCE,
					  ID_ATTRIBUTE).c_str());
  }
}

#endif /* CEXFILE_OBJECT_LABEL_H */
