/*       -*- C++ -*- */
/** \file
 ********************************************************************
 * $URL: svn+ssh://nalw02/home/alg/schlemme/svn/cextools/trunk/include/cexfile.h $
 * $Id: cexfile.h 9 2008-02-21 13:50:28Z schlemme $
 * $Date: 2008-02-21 14:50:28 +0100 (Do, 21 Feb 2008) $
 * $Revision: 9 $
 * \author Tobias Schlemmer <keinstein@users.berlios.de>
 * $Author: schlemme $ 
 * \license LGPL
 * 
 * PDF Plotting textpane class. 
 *
 * \addtogroup cairoplotter
 *
 *\{
 ********************************************************************
 */	  
#ifndef TEXTPANE_H
#define TEXTPANE_H
#include "cexfile/rect.h"
#include <cairomm/context.h>
#include <cairomm/surface.h>

/// PDF plotting class
/** This class is used to plot the diagrams into a multipage PDF file */
class textpane: protected Cairo::TextExtents {
private:
  double baselineskip; ///< baseline difference.
  typedef std::vector<Glib::ustring> text_s;  ///< Type for Text lines
  text_s text;///< Text lines of the rectangle
  cexfile::rect area; ///< Area, which the label will use
  cexfile::point start; ///< Basepoint for first text line
  Cairo::RefPtr<Cairo::Context> context;

public:
  /// default constructor (needed for STL Containers)
  textpane():Cairo::TextExtents(),text() {} 

  /// standard constructor
  /** This constructor is normally used to create a textpane object.

      \param t Vector of Text lines
      \param center Anchor point of the pane (center of the pane)
      in Cairo coordinates
      \param c Device context, where the textpane shall be located.
  */
  textpane(text_s t, 
	   cexfile::point center,
	   Cairo::RefPtr<Cairo::Context> c);

  /// return the coordinates of the rectangle
  /** This function returns the coordinates of the rectangle
      which will be needed to draw the text.
      \return rectangle as it will be drawn in current coordinates
  */
  const cexfile::rect & get_extents() const { return area; }


  /// plot the text
  /** This function will plot the text pane on the context.
   */
  void plot(Cairo::RefPtr<Cairo::Context> cxt);
	
};

/// a set of textpane objects
typedef std::list<textpane> textpane_s;

/// add one textpane to textpane_s
/** This operator appends one textpane to the list 
    \relates textpane_s
    \relatesalso textpane
    \param s set of textpanes
    \param p textpane to be added
    \return reference to the set after adding the pane
*/
inline textpane_s & operator += (textpane_s & s, const textpane & p) {
  s.push_back(p);
  return s;
}

#endif /* TEXTPANE_H */
/** \} */
